@@ -4,38 +4,35 @@ use std::path::Path;
 
 use ruff_notebook::CellOffsets;
 use ruff_python_ast::PySourceType;
+use ruff_python_ast::token::Tokens;
 use ruff_python_codegen::Stylist;
-
-use ruff_diagnostics::Diagnostic;
 use ruff_python_index::Indexer;
-use ruff_python_parser::Tokens;
-use ruff_source_file::Locator;
-use ruff_text_size::Ranged;
 
+use crate::Locator;
 use crate::directives::TodoComment;
-use crate::registry::{AsRule, Rule};
+use crate::registry::Rule;
 use crate::rules::pycodestyle::rules::BlankLinesChecker;
 use crate::rules::{
     eradicate, flake8_commas, flake8_executable, flake8_fixme, flake8_implicit_str_concat,
     flake8_pyi, flake8_todos, pycodestyle, pygrep_hooks, pylint, pyupgrade, ruff,
 };
-use crate::settings::LinterSettings;
 
-#[allow(clippy::too_many_arguments)]
+use super::ast::LintContext;
+
+#[expect(clippy::too_many_arguments)]
 pub(crate) fn check_tokens(
     tokens: &Tokens,
     path: &Path,
     locator: &Locator,
     indexer: &Indexer,
     stylist: &Stylist,
-    settings: &LinterSettings,
     source_type: PySourceType,
     cell_offsets: Option<&CellOffsets>,
-) -> Vec<Diagnostic> {
-    let mut diagnostics: Vec<Diagnostic> = vec![];
+    context: &mut LintContext,
+) {
     let comment_ranges = indexer.comment_ranges();
 
-    if settings.rules.any_enabled(&[
+    if context.any_rule_enabled(&[
         Rule::BlankLineBetweenMethods,
         Rule::BlankLinesTopLevel,
         Rule::TooManyBlankLines,
@@ -43,50 +40,37 @@ pub(crate) fn check_tokens(
         Rule::BlankLinesAfterFunctionOrClass,
         Rule::BlankLinesBeforeNestedDefinition,
     ]) {
-        BlankLinesChecker::new(locator, stylist, settings, source_type, cell_offsets)
-            .check_lines(tokens, &mut diagnostics);
+        BlankLinesChecker::new(locator, stylist, source_type, cell_offsets, context)
+            .check_lines(tokens);
     }
 
-    if settings.rules.enabled(Rule::BlanketTypeIgnore) {
-        pygrep_hooks::rules::blanket_type_ignore(&mut diagnostics, comment_ranges, locator);
+    if context.is_rule_enabled(Rule::BlanketTypeIgnore) {
+        pygrep_hooks::rules::blanket_type_ignore(context, comment_ranges, locator);
     }
 
-    if settings.rules.enabled(Rule::EmptyComment) {
-        pylint::rules::empty_comments(&mut diagnostics, comment_ranges, locator);
+    if context.is_rule_enabled(Rule::EmptyComment) {
+        pylint::rules::empty_comments(context, comment_ranges, locator, indexer);
     }
 
-    if settings
-        .rules
-        .enabled(Rule::AmbiguousUnicodeCharacterComment)
-    {
+    if context.is_rule_enabled(Rule::AmbiguousUnicodeCharacterComment) {
         for range in comment_ranges {
-            ruff::rules::ambiguous_unicode_character_comment(
-                &mut diagnostics,
-                locator,
-                *range,
-                settings,
-            );
+            ruff::rules::ambiguous_unicode_character_comment(context, locator, range);
         }
     }
 
-    if settings.rules.enabled(Rule::CommentedOutCode) {
-        eradicate::rules::commented_out_code(&mut diagnostics, locator, comment_ranges, settings);
+    if context.is_rule_enabled(Rule::CommentedOutCode) {
+        eradicate::rules::commented_out_code(context, locator, comment_ranges);
     }
 
-    if settings.rules.enabled(Rule::UTF8EncodingDeclaration) {
-        pyupgrade::rules::unnecessary_coding_comment(
-            &mut diagnostics,
-            locator,
-            indexer,
-            comment_ranges,
-        );
+    if context.is_rule_enabled(Rule::UTF8EncodingDeclaration) {
+        pyupgrade::rules::unnecessary_coding_comment(context, locator, comment_ranges);
     }
 
-    if settings.rules.enabled(Rule::TabIndentation) {
-        pycodestyle::rules::tab_indentation(&mut diagnostics, locator, indexer);
+    if context.is_rule_enabled(Rule::TabIndentation) {
+        pycodestyle::rules::tab_indentation(context, locator, indexer);
     }
 
-    if settings.rules.any_enabled(&[
+    if context.any_rule_enabled(&[
         Rule::InvalidCharacterBackspace,
         Rule::InvalidCharacterSub,
         Rule::InvalidCharacterEsc,
@@ -94,22 +78,17 @@ pub(crate) fn check_tokens(
         Rule::InvalidCharacterZeroWidthSpace,
     ]) {
         for token in tokens {
-            pylint::rules::invalid_string_characters(
-                &mut diagnostics,
-                token.kind(),
-                token.range(),
-                locator,
-            );
+            pylint::rules::invalid_string_characters(context, token, locator);
         }
     }
 
-    if settings.rules.any_enabled(&[
+    if context.any_rule_enabled(&[
         Rule::MultipleStatementsOnOneLineColon,
         Rule::MultipleStatementsOnOneLineSemicolon,
         Rule::UselessSemicolon,
     ]) {
         pycodestyle::rules::compound_statements(
-            &mut diagnostics,
+            context,
             tokens,
             locator,
             indexer,
@@ -118,46 +97,40 @@ pub(crate) fn check_tokens(
         );
     }
 
-    if settings.rules.any_enabled(&[
+    if context.any_rule_enabled(&[
         Rule::SingleLineImplicitStringConcatenation,
         Rule::MultiLineImplicitStringConcatenation,
     ]) {
-        flake8_implicit_str_concat::rules::implicit(
-            &mut diagnostics,
-            tokens,
-            settings,
-            locator,
-            indexer,
-        );
+        flake8_implicit_str_concat::rules::implicit(context, tokens, locator, indexer);
     }
 
-    if settings.rules.any_enabled(&[
+    if context.any_rule_enabled(&[
         Rule::MissingTrailingComma,
         Rule::TrailingCommaOnBareTuple,
         Rule::ProhibitedTrailingComma,
     ]) {
-        flake8_commas::rules::trailing_commas(&mut diagnostics, tokens, locator, indexer);
+        flake8_commas::rules::trailing_commas(context, tokens, locator, indexer);
     }
 
-    if settings.rules.enabled(Rule::ExtraneousParentheses) {
-        pyupgrade::rules::extraneous_parentheses(&mut diagnostics, tokens, locator);
+    if context.is_rule_enabled(Rule::ExtraneousParentheses) {
+        pyupgrade::rules::extraneous_parentheses(context, tokens, locator);
     }
 
-    if source_type.is_stub() && settings.rules.enabled(Rule::TypeCommentInStub) {
-        flake8_pyi::rules::type_comment_in_stub(&mut diagnostics, locator, comment_ranges);
+    if source_type.is_stub() && context.is_rule_enabled(Rule::TypeCommentInStub) {
+        flake8_pyi::rules::type_comment_in_stub(context, locator, comment_ranges);
     }
 
-    if settings.rules.any_enabled(&[
+    if context.any_rule_enabled(&[
         Rule::ShebangNotExecutable,
         Rule::ShebangMissingExecutableFile,
         Rule::ShebangLeadingWhitespace,
         Rule::ShebangNotFirstLine,
         Rule::ShebangMissingPython,
     ]) {
-        flake8_executable::rules::from_tokens(&mut diagnostics, path, locator, comment_ranges);
+        flake8_executable::rules::from_tokens(context, path, locator, comment_ranges);
     }
 
-    if settings.rules.any_enabled(&[
+    if context.any_rule_enabled(&[
         Rule::InvalidTodoTag,
         Rule::MissingTodoAuthor,
         Rule::MissingTodoLink,
@@ -178,15 +151,11 @@ pub(crate) fn check_tokens(
                 TodoComment::from_comment(comment, *comment_range, i)
             })
             .collect();
-        flake8_todos::rules::todos(&mut diagnostics, &todo_comments, locator, comment_ranges);
-        flake8_fixme::rules::todos(&mut diagnostics, &todo_comments);
+        flake8_todos::rules::todos(context, &todo_comments, locator, comment_ranges);
+        flake8_fixme::rules::todos(context, &todo_comments);
     }
 
-    if settings.rules.enabled(Rule::TooManyNewlinesAtEndOfFile) {
-        pycodestyle::rules::too_many_newlines_at_end_of_file(&mut diagnostics, tokens);
+    if context.is_rule_enabled(Rule::TooManyNewlinesAtEndOfFile) {
+        pycodestyle::rules::too_many_newlines_at_end_of_file(context, tokens, cell_offsets);
     }
-
-    diagnostics.retain(|diagnostic| settings.rules.enabled(diagnostic.kind.rule()));
-
-    diagnostics
 }
