@@ -4,7 +4,9 @@ use rustc_hash::FxHashSet;
 use std::sync::LazyLock;
 
 use ruff_macros::{ViolationMetadata, derive_message_formats};
+use ruff_python_ast::Parameter;
 use ruff_python_ast::docstrings::{clean_space, leading_space};
+use ruff_python_ast::helpers::map_subscript;
 use ruff_python_ast::identifier::Identifier;
 use ruff_python_semantic::analyze::visibility::is_staticmethod;
 use ruff_python_trivia::textwrap::dedent;
@@ -89,6 +91,7 @@ use crate::{Edit, Fix};
 /// - [NumPy Style Guide](https://numpydoc.readthedocs.io/en/latest/format.html)
 /// - [Google Python Style Guide - Docstrings](https://google.github.io/styleguide/pyguide.html#38-comments-and-docstrings)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.73")]
 pub(crate) struct OverindentedSection {
     name: String,
 }
@@ -192,6 +195,7 @@ impl AlwaysFixableViolation for OverindentedSection {
 /// - [PEP 287 – reStructuredText Docstring Format](https://peps.python.org/pep-0287/)
 /// - [NumPy Style Guide](https://numpydoc.readthedocs.io/en/latest/format.html)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.73")]
 pub(crate) struct OverindentedSectionUnderline {
     name: String,
 }
@@ -275,6 +279,7 @@ impl AlwaysFixableViolation for OverindentedSectionUnderline {
 /// - [NumPy Style Guide](https://numpydoc.readthedocs.io/en/latest/format.html)
 /// - [Google Python Style Guide - Docstrings](https://google.github.io/styleguide/pyguide.html#38-comments-and-docstrings)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.71")]
 pub(crate) struct NonCapitalizedSectionName {
     name: String,
 }
@@ -373,6 +378,7 @@ impl AlwaysFixableViolation for NonCapitalizedSectionName {
 /// - [PEP 287 – reStructuredText Docstring Format](https://peps.python.org/pep-0287/)
 /// - [NumPy Style Guide](https://numpydoc.readthedocs.io/en/latest/format.html)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.71")]
 pub(crate) struct MissingNewLineAfterSectionName {
     name: String,
 }
@@ -476,6 +482,7 @@ impl AlwaysFixableViolation for MissingNewLineAfterSectionName {
 /// - [PEP 287 – reStructuredText Docstring Format](https://peps.python.org/pep-0287/)
 /// - [NumPy Style Guide](https://numpydoc.readthedocs.io/en/latest/format.html)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.71")]
 pub(crate) struct MissingDashedUnderlineAfterSection {
     name: String,
 }
@@ -582,6 +589,7 @@ impl AlwaysFixableViolation for MissingDashedUnderlineAfterSection {
 /// - [PEP 287 – reStructuredText Docstring Format](https://peps.python.org/pep-0287/)
 /// - [NumPy Style Guide](https://numpydoc.readthedocs.io/en/latest/format.html)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.71")]
 pub(crate) struct MissingSectionUnderlineAfterName {
     name: String,
 }
@@ -686,6 +694,7 @@ impl AlwaysFixableViolation for MissingSectionUnderlineAfterName {
 /// - [PEP 287 – reStructuredText Docstring Format](https://peps.python.org/pep-0287/)
 /// - [NumPy Style Guide](https://numpydoc.readthedocs.io/en/latest/format.html)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.71")]
 pub(crate) struct MismatchedSectionUnderlineLength {
     name: String,
 }
@@ -783,6 +792,7 @@ impl AlwaysFixableViolation for MismatchedSectionUnderlineLength {
 /// - [NumPy Style Guide](https://numpydoc.readthedocs.io/en/latest/format.html)
 /// - [Google Style Guide](https://google.github.io/styleguide/pyguide.html#38-comments-and-docstrings)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.71")]
 pub(crate) struct NoBlankLineAfterSection {
     name: String,
 }
@@ -876,6 +886,7 @@ impl AlwaysFixableViolation for NoBlankLineAfterSection {
 /// - [PEP 287 – reStructuredText Docstring Format](https://peps.python.org/pep-0287/)
 /// - [NumPy Style Guide](https://numpydoc.readthedocs.io/en/latest/format.html)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.71")]
 pub(crate) struct NoBlankLineBeforeSection {
     name: String,
 }
@@ -971,6 +982,7 @@ impl AlwaysFixableViolation for NoBlankLineBeforeSection {
 /// - [PEP 287 – reStructuredText Docstring Format](https://peps.python.org/pep-0287/)
 /// - [NumPy Style Guide](https://numpydoc.readthedocs.io/en/latest/format.html)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.71")]
 pub(crate) struct MissingBlankLineAfterLastSection {
     name: String,
 }
@@ -1060,6 +1072,7 @@ impl AlwaysFixableViolation for MissingBlankLineAfterLastSection {
 /// - [NumPy Style Guide](https://numpydoc.readthedocs.io/en/latest/format.html)
 /// - [Google Style Guide](https://google.github.io/styleguide/pyguide.html#38-comments-and-docstrings)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.71")]
 pub(crate) struct EmptyDocstringSection {
     name: String,
 }
@@ -1137,6 +1150,7 @@ impl Violation for EmptyDocstringSection {
 /// - [PEP 287 – reStructuredText Docstring Format](https://peps.python.org/pep-0287/)
 /// - [Google Style Guide](https://google.github.io/styleguide/pyguide.html#38-comments-and-docstrings)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.74")]
 pub(crate) struct MissingSectionNameColon {
     name: String,
 }
@@ -1172,6 +1186,9 @@ impl AlwaysFixableViolation for MissingSectionNameColon {
 /// This rule is enabled when using the `google` convention, and disabled when
 /// using the `pep257` and `numpy` conventions.
 ///
+/// Parameters annotated with `typing.Unpack` are exempt from this rule.
+/// This follows the Python typing specification for unpacking keyword arguments.
+///
 /// ## Example
 /// ```python
 /// def calculate_speed(distance: float, time: float) -> float:
@@ -1221,7 +1238,9 @@ impl AlwaysFixableViolation for MissingSectionNameColon {
 /// - [PEP 257 – Docstring Conventions](https://peps.python.org/pep-0257/)
 /// - [PEP 287 – reStructuredText Docstring Format](https://peps.python.org/pep-0287/)
 /// - [Google Python Style Guide - Docstrings](https://google.github.io/styleguide/pyguide.html#38-comments-and-docstrings)
+/// - [Python - Unpack for keyword arguments](https://typing.python.org/en/latest/spec/callables.html#unpack-kwargs)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.73")]
 pub(crate) struct UndocumentedParam {
     /// The name of the function being documented.
     definition: String,
@@ -1304,6 +1323,7 @@ impl Violation for UndocumentedParam {
 /// - [NumPy Style Guide](https://numpydoc.readthedocs.io/en/latest/format.html)
 /// - [Google Python Style Guide - Docstrings](https://google.github.io/styleguide/pyguide.html#38-comments-and-docstrings)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.71")]
 pub(crate) struct BlankLinesBetweenHeaderAndContent {
     name: String,
 }
@@ -1361,7 +1381,7 @@ fn blanks_and_section_underline(
     if let Some(non_blank_line) = following_lines.next() {
         if let Some(dashed_line) = find_underline(&non_blank_line, '-') {
             if num_blank_lines_after_header > 0 {
-                if checker.enabled(Rule::MissingSectionUnderlineAfterName) {
+                if checker.is_rule_enabled(Rule::MissingSectionUnderlineAfterName) {
                     let mut diagnostic = checker.report_diagnostic(
                         MissingSectionUnderlineAfterName {
                             name: context.section_name().to_string(),
@@ -1378,7 +1398,7 @@ fn blanks_and_section_underline(
             }
 
             if dashed_line.len().to_usize() != context.section_name().len() {
-                if checker.enabled(Rule::MismatchedSectionUnderlineLength) {
+                if checker.is_rule_enabled(Rule::MismatchedSectionUnderlineLength) {
                     let mut diagnostic = checker.report_diagnostic(
                         MismatchedSectionUnderlineLength {
                             name: context.section_name().to_string(),
@@ -1394,7 +1414,7 @@ fn blanks_and_section_underline(
                 }
             }
 
-            if checker.enabled(Rule::OverindentedSectionUnderline) {
+            if checker.is_rule_enabled(Rule::OverindentedSectionUnderline) {
                 let leading_space = leading_space(&non_blank_line);
                 let docstring_indentation = docstring.compute_indentation();
                 if leading_space.len() > docstring_indentation.len() {
@@ -1434,15 +1454,13 @@ fn blanks_and_section_underline(
                     }
 
                     if following_lines.peek().is_none() {
-                        if checker.enabled(Rule::EmptyDocstringSection) {
-                            checker.report_diagnostic(
-                                EmptyDocstringSection {
-                                    name: context.section_name().to_string(),
-                                },
-                                context.section_name_range(),
-                            );
-                        }
-                    } else if checker.enabled(Rule::BlankLinesBetweenHeaderAndContent) {
+                        checker.report_diagnostic_if_enabled(
+                            EmptyDocstringSection {
+                                name: context.section_name().to_string(),
+                            },
+                            context.section_name_range(),
+                        );
+                    } else if checker.is_rule_enabled(Rule::BlankLinesBetweenHeaderAndContent) {
                         // If the section is followed by exactly one line, and then a
                         // reStructuredText directive, the blank lines should be preserved, as in:
                         //
@@ -1495,17 +1513,16 @@ fn blanks_and_section_underline(
                     }
                 }
             } else {
-                if checker.enabled(Rule::EmptyDocstringSection) {
-                    checker.report_diagnostic(
-                        EmptyDocstringSection {
-                            name: context.section_name().to_string(),
-                        },
-                        context.section_name_range(),
-                    );
-                }
+                checker.report_diagnostic_if_enabled(
+                    EmptyDocstringSection {
+                        name: context.section_name().to_string(),
+                    },
+                    context.section_name_range(),
+                );
             }
         } else {
-            if style.is_numpy() && checker.enabled(Rule::MissingDashedUnderlineAfterSection) {
+            if style.is_numpy() && checker.is_rule_enabled(Rule::MissingDashedUnderlineAfterSection)
+            {
                 if let Some(equal_line) = find_underline(&non_blank_line, '=') {
                     let mut diagnostic = checker.report_diagnostic(
                         MissingDashedUnderlineAfterSection {
@@ -1542,7 +1559,7 @@ fn blanks_and_section_underline(
                 }
             }
             if num_blank_lines_after_header > 0 {
-                if checker.enabled(Rule::BlankLinesBetweenHeaderAndContent) {
+                if checker.is_rule_enabled(Rule::BlankLinesBetweenHeaderAndContent) {
                     // If the section is followed by exactly one line, and then a
                     // reStructuredText directive, the blank lines should be preserved, as in:
                     //
@@ -1597,7 +1614,7 @@ fn blanks_and_section_underline(
         }
     } else {
         // Nothing but blank lines after the section header.
-        if style.is_numpy() && checker.enabled(Rule::MissingDashedUnderlineAfterSection) {
+        if style.is_numpy() && checker.is_rule_enabled(Rule::MissingDashedUnderlineAfterSection) {
             let mut diagnostic = checker.report_diagnostic(
                 MissingDashedUnderlineAfterSection {
                     name: context.section_name().to_string(),
@@ -1617,14 +1634,12 @@ fn blanks_and_section_underline(
                 context.summary_range().end(),
             )));
         }
-        if checker.enabled(Rule::EmptyDocstringSection) {
-            checker.report_diagnostic(
-                EmptyDocstringSection {
-                    name: context.section_name().to_string(),
-                },
-                context.section_name_range(),
-            );
-        }
+        checker.report_diagnostic_if_enabled(
+            EmptyDocstringSection {
+                name: context.section_name().to_string(),
+            },
+            context.section_name_range(),
+        );
     }
 }
 
@@ -1635,7 +1650,7 @@ fn common_section(
     next: Option<&SectionContext>,
     style: SectionStyle,
 ) {
-    if checker.enabled(Rule::NonCapitalizedSectionName) {
+    if checker.is_rule_enabled(Rule::NonCapitalizedSectionName) {
         let capitalized_section_name = context.kind().as_str();
         if context.section_name() != capitalized_section_name {
             let section_range = context.section_name_range();
@@ -1654,7 +1669,7 @@ fn common_section(
         }
     }
 
-    if checker.enabled(Rule::OverindentedSection) {
+    if checker.is_rule_enabled(Rule::OverindentedSection) {
         let leading_space = leading_space(context.summary_line());
         let docstring_indentation = docstring.compute_indentation();
         if leading_space.len() > docstring_indentation.len() {
@@ -1680,7 +1695,7 @@ fn common_section(
     let line_end = checker.stylist().line_ending().as_str();
 
     if let Some(next) = next {
-        if checker.enabled(Rule::NoBlankLineAfterSection) {
+        if checker.is_rule_enabled(Rule::NoBlankLineAfterSection) {
             let num_blank_lines = context
                 .following_lines()
                 .rev()
@@ -1704,7 +1719,7 @@ fn common_section(
     } else {
         // The first blank line is the line containing the closing triple quotes, so we need at
         // least two.
-        if checker.enabled(Rule::MissingBlankLineAfterLastSection) {
+        if checker.is_rule_enabled(Rule::MissingBlankLineAfterLastSection) {
             let num_blank_lines = context
                 .following_lines()
                 .rev()
@@ -1722,7 +1737,7 @@ fn common_section(
                     format!(
                         "{}{}",
                         line_end.repeat(2 - num_blank_lines),
-                        docstring.compute_indentation()
+                        leading_space(docstring.compute_indentation())
                     ),
                     context.end() - del_len,
                     context.end(),
@@ -1740,7 +1755,7 @@ fn common_section(
         }
     }
 
-    if checker.enabled(Rule::NoBlankLineBeforeSection) {
+    if checker.is_rule_enabled(Rule::NoBlankLineBeforeSection) {
         if !context
             .previous_line()
             .is_some_and(|line| line.trim().is_empty())
@@ -1788,7 +1803,7 @@ fn missing_args(checker: &Checker, docstring: &Docstring, docstrings_args: &FxHa
 
     // Check specifically for `vararg` and `kwarg`, which can be prefixed with a
     // single or double star, respectively.
-    if !checker.settings.pydocstyle.ignore_var_parameters() {
+    if !checker.settings().pydocstyle.ignore_var_parameters() {
         if let Some(arg) = function.parameters.vararg.as_ref() {
             let arg_name = arg.name.as_str();
             let starred_arg_name = format!("*{arg_name}");
@@ -1799,7 +1814,9 @@ fn missing_args(checker: &Checker, docstring: &Docstring, docstrings_args: &FxHa
                 missing_arg_names.insert(starred_arg_name);
             }
         }
-        if let Some(arg) = function.parameters.kwarg.as_ref() {
+        if let Some(arg) = function.parameters.kwarg.as_ref()
+            && !has_unpack_annotation(checker, arg)
+        {
             let arg_name = arg.name.as_str();
             let starred_arg_name = format!("**{arg_name}");
             if !arg_name.starts_with('_')
@@ -1825,6 +1842,15 @@ fn missing_args(checker: &Checker, docstring: &Docstring, docstrings_args: &FxHa
     }
 }
 
+/// Returns `true` if the parameter is annotated with `typing.Unpack`
+fn has_unpack_annotation(checker: &Checker, parameter: &Parameter) -> bool {
+    parameter.annotation.as_ref().is_some_and(|annotation| {
+        checker
+            .semantic()
+            .match_typing_expr(map_subscript(annotation), "Unpack")
+    })
+}
+
 // See: `GOOGLE_ARGS_REGEX` in `pydocstyle/checker.py`.
 static GOOGLE_ARGS_REGEX: LazyLock<Regex> =
     LazyLock::new(|| Regex::new(r"^\s*(\*?\*?\w+)\s*(\(.*?\))?\s*:(\r\n|\n)?\s*.+").unwrap());
@@ -1930,7 +1956,7 @@ fn numpy_section(
 ) {
     common_section(checker, docstring, context, next, SectionStyle::Numpy);
 
-    if checker.enabled(Rule::MissingNewLineAfterSectionName) {
+    if checker.is_rule_enabled(Rule::MissingNewLineAfterSectionName) {
         let suffix = context.summary_after_section_name();
 
         if !suffix.is_empty() {
@@ -1948,7 +1974,7 @@ fn numpy_section(
         }
     }
 
-    if checker.enabled(Rule::UndocumentedParam) {
+    if checker.is_rule_enabled(Rule::UndocumentedParam) {
         if matches!(context.kind(), SectionKind::Parameters) {
             parameters_section(checker, docstring, context);
         }
@@ -1963,7 +1989,7 @@ fn google_section(
 ) {
     common_section(checker, docstring, context, next, SectionStyle::Google);
 
-    if checker.enabled(Rule::MissingSectionNameColon) {
+    if checker.is_rule_enabled(Rule::MissingSectionNameColon) {
         let suffix = context.summary_after_section_name();
         if suffix != ":" {
             let mut diagnostic = checker.report_diagnostic(
@@ -2003,7 +2029,7 @@ fn parse_google_sections(
         google_section(checker, docstring, &context, iterator.peek());
     }
 
-    if checker.enabled(Rule::UndocumentedParam) {
+    if checker.is_rule_enabled(Rule::UndocumentedParam) {
         let mut has_args = false;
         let mut documented_args: FxHashSet<String> = FxHashSet::default();
         for section_context in section_contexts {
