@@ -1,5 +1,6 @@
 use ruff_macros::{ViolationMetadata, derive_message_formats};
 
+use itertools::Itertools;
 use ruff_python_ast::{self as ast, Expr};
 use ruff_python_trivia::PythonWhitespace;
 use ruff_text_size::Ranged;
@@ -47,6 +48,7 @@ use crate::{Edit, Fix, FixAvailability, Violation};
 /// ## References
 /// - [Python documentation: `decimal`](https://docs.python.org/3/library/decimal.html)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "0.12.0")]
 pub(crate) struct VerboseDecimalConstructor {
     replacement: String,
 }
@@ -90,18 +92,25 @@ pub(crate) fn verbose_decimal_constructor(checker: &Checker, call: &ast::ExprCal
             // using this regex:
             // https://github.com/python/cpython/blob/ac556a2ad1213b8bb81372fe6fb762f5fcb076de/Lib/_pydecimal.py#L6060-L6077
             // _after_ trimming whitespace from the string and removing all occurrences of "_".
-            let mut trimmed = Cow::from(str_literal.to_str().trim_whitespace());
-            if memchr::memchr(b'_', trimmed.as_bytes()).is_some() {
-                trimmed = Cow::from(trimmed.replace('_', ""));
-            }
+            let original_str = str_literal.to_str().trim_whitespace();
+            // Strip leading underscores before extracting the sign, as Python's Decimal parser
+            // removes underscores before parsing the sign.
+            let sign_check_str = original_str.trim_start_matches('_');
             // Extract the unary sign, if any.
-            let (unary, rest) = if let Some(trimmed) = trimmed.strip_prefix('+') {
-                ("+", Cow::from(trimmed))
-            } else if let Some(trimmed) = trimmed.strip_prefix('-') {
-                ("-", Cow::from(trimmed))
+            let (unary, sign_check_str) = if let Some(trimmed) = sign_check_str.strip_prefix('+') {
+                ("+", trimmed)
+            } else if let Some(trimmed) = sign_check_str.strip_prefix('-') {
+                ("-", trimmed)
             } else {
-                ("", trimmed)
+                ("", sign_check_str)
             };
+            // Save the string after the sign for normalization (before removing underscores)
+            let str_after_sign_for_normalization = sign_check_str;
+            let mut rest = Cow::from(sign_check_str);
+            let has_digit_separators = memchr::memchr(b'_', original_str.as_bytes()).is_some();
+            if has_digit_separators {
+                rest = Cow::from(rest.replace('_', ""));
+            }
 
             // Early return if we now have an empty string
             // or a very long string:
@@ -117,6 +126,13 @@ pub(crate) fn verbose_decimal_constructor(checker: &Checker, call: &ast::ExprCal
                 return;
             }
 
+            // If the original string had digit separators, normalize them
+            let rest = if has_digit_separators {
+                Cow::from(normalize_digit_separators(str_after_sign_for_normalization))
+            } else {
+                Cow::from(rest)
+            };
+
             // If all the characters are zeros, then the value is zero.
             let rest = match (unary, rest.is_empty()) {
                 // `Decimal("-0")` is not the same as `Decimal("0")`
@@ -125,10 +141,11 @@ pub(crate) fn verbose_decimal_constructor(checker: &Checker, call: &ast::ExprCal
                     return;
                 }
                 (_, true) => "0",
-                _ => rest,
+                _ => &rest,
             };
 
             let replacement = format!("{unary}{rest}");
+
             let mut diagnostic = checker.report_diagnostic(
                 VerboseDecimalConstructor {
                     replacement: replacement.clone(),
@@ -185,6 +202,22 @@ pub(crate) fn verbose_decimal_constructor(checker: &Checker, call: &ast::ExprCal
     }
 }
 
+/// Normalizes digit separators in a numeric string by:
+/// - Stripping leading and trailing underscores
+/// - Collapsing medial underscore sequences to single underscores
+fn normalize_digit_separators(original_str: &str) -> String {
+    // Strip leading and trailing underscores
+    let trimmed = original_str
+        .trim_start_matches(['_', '0'])
+        .trim_end_matches('_');
+
+    // Collapse medial underscore sequences to single underscores
+    trimmed
+        .chars()
+        .dedup_by(|a, b| *a == '_' && a == b)
+        .collect()
+}
+
 // ```console
 // $ python
 // >>> import sys
