@@ -9,11 +9,14 @@
 
 from __future__ import annotations
 
+import argparse
 import json
 import os
 import subprocess
+import sys
+import threading
 from pathlib import Path
-from typing import Final, Literal, Never, assert_never
+from typing import Final, Literal, assert_never
 
 from rich.console import Console
 from watchfiles import Change, watch
@@ -27,15 +30,23 @@ DIRS_TO_WATCH: Final = (
     CRATE_ROOT.parent / "ty_test/src",
 )
 MDTEST_DIR: Final = CRATE_ROOT / "resources" / "mdtest"
+MDTEST_README: Final = CRATE_ROOT / "resources" / "README.md"
 
 
 class MDTestRunner:
     mdtest_executable: Path | None
     console: Console
+    filters: list[str]
+    enable_external: bool
 
-    def __init__(self) -> None:
+    def __init__(self, filters: list[str] | None, enable_external: bool) -> None:
         self.mdtest_executable = None
         self.console = Console()
+        self.filters = [
+            f.removesuffix(".md").replace("/", "_").replace("-", "_")
+            for f in (filters or [])
+        ]
+        self.enable_external = enable_external
 
     def _run_cargo_test(self, *, message_format: Literal["human", "json"]) -> str:
         return subprocess.check_output(
@@ -106,19 +117,28 @@ class MDTestRunner:
         return subprocess.run(
             [self.mdtest_executable, *arguments],
             cwd=CRATE_ROOT,
-            env=dict(os.environ, CLICOLOR_FORCE="1"),
+            env=dict(
+                os.environ,
+                CLICOLOR_FORCE="1",
+                INSTA_FORCE_PASS="1",
+                INSTA_OUTPUT="none",
+                MDTEST_EXTERNAL="1" if self.enable_external else "0",
+            ),
             capture_output=capture_output,
             text=True,
             check=False,
         )
 
-    def _run_mdtests_for_file(self, markdown_file: Path) -> None:
-        path_mangled = (
+    def _mangle_path(self, markdown_file: Path) -> str:
+        return (
             markdown_file.as_posix()
             .replace("/", "_")
             .replace("-", "_")
             .removesuffix(".md")
         )
+
+    def _run_mdtests_for_file(self, markdown_file: Path) -> None:
+        path_mangled = self._mangle_path(markdown_file)
         test_name = f"mdtest__{path_mangled}"
 
         output = self._run_mdtest(["--exact", test_name], capture_output=True)
@@ -160,9 +180,19 @@ class MDTestRunner:
 
             print(line)
 
-    def watch(self) -> Never:
+    def watch(self):
+        def keyboard_input() -> None:
+            for _ in sys.stdin:
+                # This is silly, but there is no other way to inject events into
+                # the main `watch` loop. We use changes to the `README.md` file
+                # as a trigger to re-run all mdtests:
+                MDTEST_README.touch()
+
+        input_thread = threading.Thread(target=keyboard_input, daemon=True)
+        input_thread.start()
+
         self._recompile_tests("Compiling tests...", message_on_success=False)
-        self._run_mdtest()
+        self._run_mdtest(self.filters)
         self.console.print("[dim]Ready to watch for changes...[/dim]")
 
         for changes in watch(*DIRS_TO_WATCH):
@@ -174,6 +204,11 @@ class MDTestRunner:
             for change, path_str in changes:
                 path = Path(path_str)
 
+                # See above: `README.md` changes trigger a full re-run of all tests
+                if path == MDTEST_README:
+                    self._run_mdtest(self.filters)
+                    continue
+
                 match path.suffix:
                     case ".rs":
                         rust_code_has_changed = True
@@ -209,12 +244,12 @@ class MDTestRunner:
 
             if rust_code_has_changed:
                 if self._recompile_tests("Rust code has changed, recompiling tests..."):
-                    self._run_mdtest()
+                    self._run_mdtest(self.filters)
             elif vendored_typeshed_has_changed:
                 if self._recompile_tests(
                     "Vendored typeshed has changed, recompiling tests..."
                 ):
-                    self._run_mdtest()
+                    self._run_mdtest(self.filters)
             elif new_md_files:
                 files = " ".join(file.as_posix() for file in new_md_files)
                 self._recompile_tests(
@@ -226,8 +261,27 @@ class MDTestRunner:
 
 
 def main() -> None:
+    parser = argparse.ArgumentParser(
+        description="A runner for Markdown-based tests for ty"
+    )
+    parser.add_argument(
+        "filters",
+        nargs="*",
+        help="Partial paths or mangled names, e.g., 'loops/for.md' or 'loops_for'",
+    )
+    parser.add_argument(
+        "--enable-external",
+        "-e",
+        action="store_true",
+        help="Enable tests with external dependencies",
+    )
+
+    args = parser.parse_args()
+
     try:
-        runner = MDTestRunner()
+        runner = MDTestRunner(
+            filters=args.filters, enable_external=args.enable_external
+        )
         runner.watch()
     except KeyboardInterrupt:
         print()
